import calendar
import glob
import os
import random
import re
import shutil
import tempfile
import time

import arrow
import numpy as np
import pandas as pd
from bs4 import BeautifulSoup
from selenium.common.exceptions import (
    NoSuchElementException,
    StaleElementReferenceException,
)

from utility_bill_scraper import (
    Timeout,
    UtilityAPI,
    convert_divs_to_df,
    format_fields,
    pdf_to_html,
)


def get_name():
    return "Kitchener Utilities"


def get_summary(soup):
    def find_seq_id(tag):
        return tag.name == "div" and tag.decode().find("SEQ-ID") >= 0

    def find_account_summary(tag):
        return tag.name == "span" and tag.decode().find("Your Account Summary") >= 0

    summary_fields = format_fields(soup.find_all(find_account_summary)[0].contents)
    summary_data = format_fields(
        soup.find_all(find_seq_id)[0].next_sibling.contents[0].contents
    )

    summary_dict = dict(zip(summary_fields[1:], summary_data))

    def find_charges(name):
        def find_matching_div(tag):
            return tag.name == "div" and tag.decode().find(name) >= 0

        tag = soup.find(find_matching_div)

        # Extract the top pixel coordinate.
        match = re.search(r"top:(?P<top>\d+)px", tag.decode())
        top = match.groups()[0]

        # Find the second div with the same top pixel coordinate.
        return format_fields(
            soup.find_all(style=re.compile("top:%spx" % top))[1].span.contents
        )[0]

    summary_dict["Water Charges"] = find_charges("Water charges")
    summary_dict["Gas Charges"] = find_charges("Gas charges")

    return summary_dict


def get_water_consumption(soup):
    def find_total_consumption(tag):
        return tag.name == "div" and tag.decode().find("Total Consumption") >= 0

    div_list = soup.find_all(find_total_consumption)

    # Find the div containing 3 fields (gas has an extra
    # 'Billing Conversion Multiplier'). Note that it is possible to have
    # more than one consumption section.

    tags = [x for x in div_list if len(format_fields(x.contents[0])) == 3]

    consumption = []

    for tag in tags:
        # Extract the top pixel coordinate.
        match = re.search(r"top:(?P<top>\d+)px", tag.decode())
        top = match.groups()[0]

        # Match all divs with the same top pixel coordinate.
        def find_matching_top(tag):
            return tag.name == "div" and tag.decode().find("top:%spx" % top) >= 0

        divs = [format_fields(x.contents[0]) for x in soup.find_all(find_matching_top)]
        consumption.append(dict(zip(divs[0], divs[2])))
    return consumption


def get_water_and_sewer_charges(soup):
    def find_water_consumption(tag):
        return (
            (tag.name == "div")
            and (tag.decode().find("Consumption") >= 0)
            and (tag.decode().find("Total Consumption") == -1)
        )

    water_div = soup.find_all(find_water_consumption)[0]
    water_type = format_fields(water_div.next_sibling.contents[0])

    result = {"Time period": water_type[0]}
    water_type = water_type[1:]

    consumption = format_fields(water_div.next_sibling.next_sibling.contents[0])
    rates = format_fields(
        water_div.next_sibling.next_sibling.next_sibling.next_sibling.contents[0]
    )
    charges = format_fields(
        water_div.next_sibling.next_sibling.next_sibling.next_sibling.next_sibling.contents[
            0
        ]
    )

    for x in range(len(water_type)):
        result[water_type[x]] = {
            "Consumption": consumption[x],
            "Rate": rates[x],
            "Charges": charges[x],
        }
    return result


def get_water_charges(soup):
    data = get_water_and_sewer_charges(soup)
    types = ["Water", "Sewer"]
    return dict(zip(types, [data[x]["Charges"] for x in types]))


def get_water_rates(soup):
    data = get_water_and_sewer_charges(soup)
    types = ["Water", "Sewer"]
    return dict(zip(types, [data[x]["Rate"] for x in types]))


def get_gas_consumption(soup):
    def find_total_consumption(tag):
        return tag.name == "div" and tag.decode().find("Total Consumption") >= 0

    div_list = soup.find_all(find_total_consumption)

    # Find divs containing 4 fields (gas has an extra
    # 'Billing Conversion Multiplier'). Note that it is possible to have
    # more than one consumption section.
    tags = [x for x in div_list if len(format_fields(x.contents[0])) > 3]

    consumption = []
    for tag in tags:
        # Extract the top pixel coordinate.
        match = re.search(r"top:(?P<top>\d+)px", tag.decode())
        top = match.groups()[0]

        # Match all divs with the same top pixel coordinate.
        def find_matching_top(tag):
            return tag.name == "div" and tag.decode().find("top:%spx" % top) >= 0

        divs = [format_fields(x.contents[0]) for x in soup.find_all(find_matching_top)]

        consumption.append(dict(zip(divs[0], divs[2])))
    return consumption


def get_gas_charges(soup):
    try:
        # Find the bounding box that defines the gas section.
        pos_re = r"left:(?P<left>\d+)px.*top:(?P<top>\d+)px.*width:(?P<width>\d+)px.*height:(?P<height>\d+)"

        def find_gas_section(tag):
            return tag.name == "div" and tag.decode().find("GAS") >= 0

        tag = soup.find(find_gas_section)
        pos = re.search(pos_re, tag.decode()).groupdict()
        top_bound = int(pos["top"])

        def find_gas_charges(tag):
            return tag.name == "div" and tag.decode().find("Gas charges") >= 0

        tag = soup.find(find_gas_charges)
        pos = re.search(pos_re, tag.decode()).groupdict()
        bottom_bound = int(pos["top"])

        # Find all of the div tags within this bounding box.
        def find_divs_within_bounds(tag):
            match = re.search(pos_re, tag.decode())
            if match:
                top = int(match.groupdict()["top"])
                return top >= top_bound and top < bottom_bound and tag.name == "div"
            return False

        df = convert_divs_to_df(soup.find_all(find_divs_within_bounds))
        df["fields_str"] = [str(x) for x in df["fields"]]
        df = df.sort_values(["top", "left"])

        # Charges can be grouped in different sections (e.g., if the gas rate
        # changes in the middle of the month). We only care about the last
        # section, because it contains the Fixed Delivery Charge.
        charges = df[
            df["left"] > df[df["fields_str"] == "[u'Charges']"]["left"].iloc[0]
        ].iloc[-1]["fields"]
        charge_desc = df[df["fields_str"].str.find(" days") >= 0].iloc[-1]["fields"][1:]
        return dict(zip(charge_desc[-len(charges) :], charges))
    except AttributeError as error:
        print("Error scraping gas charges")
    return {}


def get_gas_rates(soup):
    def find_gas_rates(tag):
        return tag.name == "div" and tag.decode().find("Gas Fixed Delivery Charge") >= 0

    gas_div = soup.find_all(find_gas_rates)[0]
    gas_fields = format_fields(gas_div.contents[0])
    gas_fields = gas_fields[1:]
    gas_rates = format_fields(
        gas_div.next_sibling.next_sibling.next_sibling.contents[0]
    )
    format_fields(
        gas_div.next_sibling.next_sibling.next_sibling.next_sibling.contents[0]
    )

    return dict(
        zip(
            [
                x + " Rate"
                for x in gas_fields
                if (x.find("HST") == -1) and x.find("Fixed") == -1
            ],
            gas_rates,
        )
    )


class KitchenerUtilitiesAPI(UtilityAPI):
    name = get_name()

    def __init__(
        self,
        user=None,
        password=None,
        data_path=None,
        file_ext=".csv",
        headless=True,
        timeout=10,
        save_statements=True,
        google_sa_credentials=None,
    ):
        super().__init__(
            user=user,
            password=password,
            data_path=data_path,
            file_ext=file_ext,
            headless=headless,
            timeout=timeout,
            save_statements=save_statements,
            google_sa_credentials=google_sa_credentials,
        )

    def _login(self):
        self._driver.get(
            "https://ebilling.kitchener.ca/sap/bc/ui5_ui5/sap/ZUMCUI5/index.html"
        )
        self._driver.find_element_by_id("CANCEL_BUTTON ").click()
        self._driver.find_element_by_id("__field1").send_keys(self._user)
        self._driver.find_element_by_id("__field0").send_keys(self._password)
        self._driver.find_element_by_id("__button0").click()

    def _get_header_nav_bar(self):
        result = None
        t_start = time.time()
        while time.time() - t_start < self._timeout:
            try:
                pages = self._driver.find_element_by_id(
                    "headerNavigationBar"
                ).find_elements_by_tag_name("li")
                keys = [x.text for x in pages]
                result = dict(zip(keys, pages))
                result.pop("", None)
                return result
            except NoSuchElementException:
                pass
        raise Timeout

    def _get_contracts(self):
        # Pick the account (e.g., "Gas", "Water and Sewer", "Stormwater")
        contract_table = self._driver.find_element_by_id(
            "ContractTable-table"
        ).find_element_by_tag_name("tbody")
        rows = contract_table.find_elements_by_tag_name("tr")

        contracts = [x.find_elements_by_tag_name("td")[0] for x in rows]
        keys = [x.text for x in contracts]
        return dict(zip(keys, contracts))

    def _first_page(self):
        # Get a list of the pages available
        t_start = time.time()
        link = None
        while time.time() - t_start < self._timeout:
            try:
                link = self._driver.find_element_by_id(
                    "__table1-paginator--firstPageLink"
                )
                link.location_once_scrolled_into_view
                link.click()
                return
            except NoSuchElementException:
                pass
            except StaleElementReferenceException:
                pass
        raise Timeout

    def _get_pages(self):
        # Get a list of the pages available
        table = None
        t_start = time.time()
        while time.time() - t_start < self._timeout:
            try:
                table = self._driver.find_element_by_id("__table1-paginator-pages")
                time.sleep(1)
                break
            except NoSuchElementException:
                pass

        if table:
            time.sleep(0.5)
            pages = table.find_elements_by_tag_name("li")
            keys = [int(x.text) for x in pages]
            return dict(zip(keys, pages))
        else:
            raise Timeout
        return None

    def download_statements(self, start_date=None, end_date=None, max_downloads=None):
        download_path = tempfile.mkdtemp()
        self._init_driver()
        downloaded_files = []

        try:
            self._login()

            # convert start and end dates to date objects
            if start_date:
                start_date = arrow.get(start_date).date()
            if end_date:
                end_date = arrow.get(end_date).date()

            def get_data():
                # Iterate through the statements in reverse chronological order
                # (i.e., newest statements are first).

                billing_table = self._driver.find_element_by_id("__table1-table")

                rows = [
                    [y for y in x.find_elements_by_tag_name("td")]
                    for x in billing_table.find_element_by_tag_name(
                        "tbody"
                    ).find_elements_by_tag_name("tr")
                ]

                data = []
                for row in rows:
                    row_data = [x.text for x in row[1:]]
                    if row_data[0] == "":
                        continue

                    date = arrow.get(row_data[1], "MM/DD/YYYY").date()

                    if start_date and date < start_date:
                        break

                    if end_date and date > end_date:
                        continue

                    if max_downloads and len(downloaded_files) >= max_downloads:
                        break

                    data.append(row_data)
                    new_filepath = os.path.join(
                        download_path,
                        "%s - %s - $%s.pdf"
                        % (date.isoformat(), self.name, row_data[3]),
                    )

                    def download_link(link, ext):
                        # remove all files in the temp dir
                        files = os.listdir(self._temp_download_dir)
                        for file in files:
                            os.remove(os.path.join(self._temp_download_dir, file))

                        # add a random delay to keep from being banned
                        time.sleep(1 * random.random() * 3)

                        # download the link
                        link.click()

                        t_start = time.time()
                        filepath = None
                        # wait for the file to finish downloading
                        while time.time() - t_start < self._timeout:
                            files = glob.glob(
                                os.path.join(self._temp_download_dir, "*.%s" % ext)
                            )
                            if len(files):
                                filepath = os.path.join(
                                    self._temp_download_dir, files[0]
                                )
                                time.sleep(0.5)
                                break
                        if not filepath:
                            raise Timeout
                        return filepath

                    downloaded_files.append(new_filepath)
                    if not os.path.exists(new_filepath):
                        # download the pdf statement
                        for img in row[0].find_elements_by_tag_name("img"):
                            if img.get_property("title") == "PDF":
                                filepath = download_link(img, "pdf")
                                shutil.move(filepath, new_filepath)

                return data

            self._get_header_nav_bar()["BILLING"].click()
            self._first_page()

            pages_downloaded = []
            pages_to_download = list(self._get_pages().keys())
            data = []
            while len(pages_to_download):
                pages = self._get_pages()

                # Add any new pages that are not in our list
                for x in pages.keys():
                    if x not in pages_to_download and x not in pages_downloaded:
                        pages_to_download += [x]

                # Download the first page in the list
                i = pages_to_download.pop(0)

                pages[i].click()
                pages_downloaded += [i]
                time.sleep(1)
                page_data = get_data()
                # If this page returned some data within the date range, add it
                if len(page_data):
                    data += page_data
                # If there was no data within the date range and we alread have data,
                # it means that any remaining pages must be prior to the start date;
                # therefore we should stop.
                elif len(data):
                    break
        finally:
            self._close_driver()

        return downloaded_files

    def get_consumption_history(self, contract):
        self._init_driver()

        try:

            def get_data():
                # The Consumption history div (this contains all of the data we are interested in)
                consumption_history = (
                    self._driver.find_element_by_id("contractConsumptionHistory")
                    .find_element_by_id("__table1-table")
                    .find_element_by_tag_name("tbody")
                )
                rows = consumption_history.find_elements_by_tag_name("tr")
                data = [
                    [y.text for y in x.find_elements_by_tag_name("td")] for x in rows
                ]
                return data

            self._login()

            self._get_header_nav_bar()["ACCOUNTS"].click()
            self._get_contracts()[contract].click()

            # Click on the "Consumption History" tab
            link = self._driver.find_element_by_id("contractDetailNavigationBarItem2")
            link.location_once_scrolled_into_view
            link.click()
            time.sleep(0.5)

            data = []
            for page in self._get_pages().keys():
                link = self._get_pages()[page]
                link.location_once_scrolled_into_view
                link.click()
                data += get_data()

            # sum values that have the same date
            dates = [x for x, y in data]
            results = dict(zip(dates, [0] * len(dates)))
            results.pop("", None)
            for date, consumption in data:
                if date != "":
                    results[date] += float(consumption)
        finally:
            self._close_driver()

        def convert_to_series(data):
            dates = [
                arrow.get("%s 01 %s" % tuple(x.split(" ")), "MMMM DD YYYY").date()
                for x in list(data.keys())
            ]
            dates = [
                "%d-%02d-%02d"
                % (x.year, x.month, calendar.monthrange(x.year, x.month)[1])
                for x in dates
            ]
            return pd.Series(list(data.values()), index=dates)

        return convert_to_series(results)

    def extract_data(self, pdf_file):
        html_file = pdf_to_html(pdf_file)
        with open(html_file, "r", encoding="utf8") as f:
            soup = BeautifulSoup(f, "html.parser")
        os.remove(html_file)
        summary = get_summary(soup)

        # To do: several functions were broken when updating to python3
        result = {
            "name": self.name,
            "summary": summary,
            "water consumption": get_water_consumption(soup),
            # 'water and sewer charges': self.get_water_and_sewer_charges(soup),
            "gas consumption": get_gas_consumption(soup),
            # 'gas charges': self.get_gas_charges(soup),
            # 'gas rates': self.get_gas_rates(soup)
        }

        if "Pre-authorized Withdrawal" in result["summary"].keys():
            amount_due = result["summary"]["Pre-authorized Withdrawal"]
        elif "Total Due" in result["summary"].keys():
            amount_due = result["summary"]["Total Due"]
        else:
            print("Couldn't find amount due!")
            amount_due = None

        new_name = "%s - %s - $%.2f.pdf" % (
            arrow.get(result["summary"]["Issue Date"], "MMM DD YYYY").format(
                "YYYY-MM-DD"
            ),
            self.name,
            amount_due,
        )
        return result, new_name

    def convert_data_to_df(self, data):
        cols = list(data[0]["summary"].keys())
        if "Pre-authorized Withdrawal" in cols:
            cols.remove("Pre-authorized Withdrawal")
            cols.append("Total Due")

        for x in data:
            if "Pre-authorized Withdrawal" in x["summary"].keys():
                x["summary"]["Total Due"] = x["summary"]["Pre-authorized Withdrawal"]
        data_sets = []
        for col in cols:
            data_sets.append([x["summary"][col] for x in data])

        df = pd.DataFrame(data=dict(zip(cols, data_sets)))

        df["Issue Date"] = [
            str(arrow.get(x, "MMM DD YYYY").date()) for x in df["Issue Date"]
        ]
        df = df.set_index("Issue Date")

        # Extract water and gas consumption.
        water_consumption = [
            np.sum(
                [
                    x["Total Consumption"] if "Total Consumption" in x else 0
                    for x in row["water consumption"]
                ]
            )
            for row in data
        ]
        gas_consumption = [
            np.sum(
                [
                    x["Total Consumption"] if "Total Consumption" in x else 0
                    for x in row["gas consumption"]
                ]
            )
            for row in data
        ]
        """
        # To do: gas charges were broken when updating to python3
        
        gas_fixed_charge = [x['gas charges']['Gas Fixed Delivery Charge']
                            if 'Gas Fixed Delivery Charge'
                            in x['gas charges'] else None for x in data]
        # Figure out the sales tax rate.
        sales_tax_on_gas = [x['gas charges']['HST on Gas']
                            if 'HST on Gas'
                            in x['gas charges'] else None for x in data]
        sales_tax_rate = df['Gas Charges'] / (
            df['Gas Charges'] - sales_tax_on_gas) - 1

        # Adjust the fixed charge by the sales tax
        df['Gas Fixed Charges'] = np.round(gas_fixed_charge * (
            1 + sales_tax_rate), 2)

        # Calculate the variable charge and rate.
        df['Gas Variable Charges'] = df['Gas Charges'] - df['Gas Fixed Charges']
        df['Gas Variable Rate'] = (df['Gas Variable Charges'] /
                                df['Gas Consumption'])
        """

        df["Water Consumption"] = water_consumption
        df["Gas Consumption"] = gas_consumption

        return df



import argparse
import os
import sys

from dotenv import load_dotenv

sys.path.insert(0, os.path.join("..", ".."))


def update(
    utility_name,
    user,
    password,
    data_path,
    save_statements,
    max_downloads,
    google_sa_credentials,
):
    if utility_name == "Kitchener Utilities":
        import utility_bill_scraper.canada.on.kitchener_utilities as ku

        api = ku.KitchenerUtilitiesAPI(
            user,
            password,
            data_path=data_path,
            save_statements=save_statements,
            google_sa_credentials=google_sa_credentials,
        )
    else:
        print(f"Unsupported utility: {utility_name}")

    updates = api.update(max_downloads=max_downloads)
    if updates is not None:
        print(f"Downloaded {len(updates)} new statements")
    else:
        print("No new updates")


def export(utility_name, data_path, output, google_sa_credentials):
    if utility_name == "Kitchener Utilities":
        import utility_bill_scraper.canada.on.kitchener_utilities as ku

        api = ku.KitchenerUtilitiesAPI(data_path=data_path)
    else:
        print(f"Unsupported utility: {utility_name}")

    df = api.cached_history()
    ext = os.path.splitext(output.lower())[-1]
    if ext == ".csv":
        df.to_csv(output)
    elif ext == ".xlsx":
        df.to_excel(output)


def main():
    parser = argparse.ArgumentParser(description="ubs (Utility bill scraper)")
    parser.add_argument("-e", "--env", help="path to .env file")
    parser.add_argument(
        "--data-path", help="folder containing the data file and statements"
    )
    parser.add_argument("--utility-name", help="name of the utility")
    parser.add_argument(
        "--google-sa-credentials", help="google service account credentials"
    )

    subparsers = parser.add_subparsers(
        title="subcommands", dest="subcommand", help="available sub-commands"
    )
    subparsers.required = True
    parser_update = subparsers.add_parser("update")
    parser_update.add_argument("-u", "--user", help="user name")
    parser_update.add_argument("-p", "--password", help="password")
    parser_update.add_argument(
        "--save-statements", help="save downloaded statements (default=True)"
    )
    parser_update.add_argument(
        "-m", "--max-downloads", help="maximum number of statements to download"
    )

    parser_export = subparsers.add_parser("export")
    parser_export.add_argument("-o", "--output", help="export file path")

    args = parser.parse_args(sys.argv[1:])

    # If the user passed a .env path, load the environment.
    if args.env:
        load_dotenv(args.env, override=True)

    utility_name = args.utility_name or os.getenv("UTILITY_NAME")
    data_path = args.data_path or os.getenv("DATA_PATH")
    google_sa_credentials = args.google_sa_credentials or os.getenv(
        "GOOGLE_SA_CREDENTIALS"
    )

    from utility_bill_scraper import is_gdrive_path

    def missing_required_arg(arg):
        sys.stderr.write(f"Error: no `{arg}` set.\n")
        parser.print_help()
        sys.exit(2)

    if is_gdrive_path(data_path) and google_sa_credentials is None:
        missing_required_arg("google-sa-credentials")

    if utility_name is None:
        missing_required_arg("utility-name")

    if args.subcommand == "update":
        user = args.user or os.getenv("USER")
        password = args.password or os.getenv("PASSWORD")

        # Default for save statements is True
        save_statements = os.getenv("SAVE_STATEMENTS", True)
        # Override with command line argument.
        if args.save_statements:
            save_statements = args.save_statements.lower() in ("true", "1")

        max_downloads = args.max_downloads or os.getenv("MAX_DOWNLOADS")
        if max_downloads:
            max_downloads = int(max_downloads)
        if user is None:
            missing_required_arg("user")
        if password is None:
            missing_required_arg("password")
        update(
            utility_name,
            user,
            password,
            data_path,
            save_statements,
            max_downloads,
            google_sa_credentials,
        )
    elif args.subcommand == "export":
        output = args.output or os.getenv("OUTPUT")
        if output is None:
            missing_required_arg("output")
        export(utility_name, data_path, output, google_sa_credentials)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['utility_bill_scraper',
 'utility_bill_scraper.bin',
 'utility_bill_scraper.canada.on']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=1.2.0,<2.0.0',
 'beautifulsoup4>=4.10.0,<5.0.0',
 'google-api-python-client>=2.27.0,<3.0.0',
 'matplotlib>=3.4.3,<4.0.0',
 'numpy>=1.21.2,<2.0.0',
 'pandas>=1.3.3,<2.0.0',
 'pdfminer>=20191125,<20191126',
 'python-dotenv>=0.19.1,<0.20.0',
 'selenium>=3.141.0,<4.0.0']

setup_kwargs = {
    'name': 'utility-bill-scraper',
    'version': '0.6.3',
    'description': 'Utility bill scraper for extracting data from pdfs and websites.',
    'long_description': '# Utility bill scraper\n\n[![build](https://github.com/ryanfobel/utility-bill-scraper/actions/workflows/build.yml/badge.svg?branch=main)](https://github.com/ryanfobel/utility-bill-scraper/actions/workflows/build.yml)\n[![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/ryanfobel/utility-bill-scraper/main)\n[![PyPI version shields.io](https://img.shields.io/pypi/v/utility-bill-scraper.svg)](https://pypi.python.org/pypi/utility-bill-scraper/)\n\nDownload energy usage data and estimate CO2 emissions from utility websites or pdf bills.\n\n<!-- START doctoc generated TOC please keep comment here to allow auto update -->\n<!-- DON\'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->\n## Table of contents\n\n- [Supported utilities](#supported-utilities)\n- [Install](#install)\n- [Data storage](#data-storage)\n- [Getting and plotting data using the Python API](#getting-and-plotting-data-using-the-python-api)\n  - [Update data](#update-data)\n  - [Plot monthly gas consumption](#plot-monthly-gas-consumption)\n  - [Convert gas consumption to CO2 emissions](#convert-gas-consumption-to-co2-emissions)\n  - [Plot CO2 emissions versus previous years](#plot-co2-emissions-versus-previous-years)\n- [Command line utilities](#command-line-utilities)\n  - [Update data](#update-data-1)\n  - [Export data](#export-data)\n  - [Options](#options)\n  - [Environment variables](#environment-variables)\n- [Contributors](#contributors)\n\n<!-- END doctoc generated TOC please keep comment here to allow auto update -->\n\n## Supported utilities\n\nThe simplest way to get started is to click on one of the following links, which will open a session on https://mybinder.org where you can try downloading some data. **Note: after you click on the link, it will take a couple of minutes to load an interactive Jupyter notebook.** Then follow the instructions (e.g., provide your `username` and `password`) to run the notebook directly from your browser.\n\n * [Kitchener Utilities (gas & water)](https://mybinder.org/v2/gh/ryanfobel/utility-bill-scraper/main?labpath=notebooks%2Fcanada%2Fon%2Fkitchener_utilities.ipynb)\n \n## Install\n\n```sh\npip install utility-bill-scraper\n```\n\n## Data storage\n\nAll data is stored in a file located at `$DATA_PATH/$UTILITY_NAME/data.csv`. The path to this file can be set as input argument when initializing an API object via the `data_path` argument.\n\n```\n└───data\n    └───Kitchener Utilities\n        └───data.csv\n        └───statements\n            │───2021-10-18 - Kitchener Utilities - $102.30.pdf\n            ...\n            └───2021-06-15 - Kitchener Utilities - $84.51.pdf\n```\n\n## Getting and plotting data using the Python API\n\n### Update data\n\n```python\nimport utility_bill_scraper.canada.on.kitchener_utilities as ku\n\nku_api = ku.KitchenerUtilitiesAPI(username=\'username\', password=\'password\')\n\n# Get new statements.\nupdates = ku_api.update()\nif updates is not None:\n    print(f"{ len(updates) } statements_downloaded")\nku_api.history().tail()\n```\n![history tail](https://raw.githubusercontent.com/ryanfobel/utility-bill-scraper/main/notebooks/canada/on/images/history_tail.png)\n\n\n\n\n### Plot monthly gas consumption\n\n```python\nimport matplotlib.pyplot as plt\n\ndf_ku = ku_api.history()\n\nplt.figure()\nplt.bar(df_ku.index, df_ku["Gas Consumption"], width=0.9, alpha=0.5)\nplt.xticks(rotation=90)\nplt.title("Monthly Gas Consumption")\nplt.ylabel("m$^3$")\n```\n\n![monthly gas consumption](https://raw.githubusercontent.com/ryanfobel/utility-bill-scraper/main/notebooks/canada/on/images/monthly_gas_consumption.png)\n\n### Convert gas consumption to CO2 emissions\n\n```python\nfrom utility_bill_scraper import GAS_KGCO2_PER_CUBIC_METER\n\ndf_ku["kgCO2"] = df_ku["Gas Consumption"] * GAS_KGCO2_PER_CUBIC_METER\n```\n\n### Plot CO2 emissions versus previous years\n\n```python\nimport datetime as dt\n\ndf_ku["kgCO2"] = df_ku["Gas Consumption"] * GAS_KGCO2_PER_CUBIC_METER\ndf_ku["year"] = [int(x[0:4]) for x in df_ku.index]\ndf_ku["month"] = [int(x[5:7]) for x in df_ku.index]\n\nn_years_history = 1\n\nplt.figure()\nfor year, df_year in df_ku.groupby("year"):\n    if year >= dt.datetime.utcnow().year - n_years_history:\n        df_year.sort_values("month", inplace=True)\n        plt.bar(\n            df_year["month"],\n            df_year["Gas Consumption"],\n            label=year,\n            width=0.9,\n            alpha=0.5,\n        )\nplt.legend()\nplt.ylabel("m$^3$")\nplt.xlabel("Month")\nylim = plt.ylim()\nax = plt.gca()\nax2 = ax.twinx()\nplt.ylabel("tCO$_2$e")\nplt.ylim([GAS_KGCO2_PER_CUBIC_METER * y / 1e3 for y in ylim])\nplt.title("Monthly CO$_2$e emissions from natural gas")\n```\n![monthly_co2_emissions](https://raw.githubusercontent.com/ryanfobel/utility-bill-scraper/main/notebooks/canada/on/images/monthly_co2_emissions.png)\n\n## Command line utilities\n\nUpdate and export your utility data from the command line.\n\n### Update data\n\n```sh\n> python -m utility_bill_scraper.bin.ubs --utilty-name "Kitchener Utilities" update --user $USER --password $PASSWORD\n```\n\n### Export data\n\n```sh\n> python -m utility_bill_scraper.bin.ubs --utilty-name "Kitchener Utilities" export --output data.csv\n```\n\n### Options\n\n```sh\n> python -m utility_bill_scraper.bin.ubs --help\nusage: ubs.py [-h] [-e ENV] [--data-path DATA_PATH] [--utility-name UTILITY_NAME]\n              [--google-sa-credentials GOOGLE_SA_CREDENTIALS]\n              {update,export} ...\n\nubs (Utility bill scraper)\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -e ENV, --env ENV     path to .env file\n  --data-path DATA_PATH\n                        folder containing the data file and statements\n  --utility-name UTILITY_NAME\n                        name of the utility\n  --google-sa-credentials GOOGLE_SA_CREDENTIALS\n                        google service account credentials\n\nsubcommands:\n  {update,export}       available sub-commands\n```\n\n### Environment variables\n\nNote that many options can be set via environment variables (useful for continuous integration and/or working with containers). The following can be set in your shell or via a `.env` file passed using the `-e` option.\n\n```sh\nDATA_PATH="folder containing the data file and statements"\nUTILITY_NAME="name of the utility"\nGOOGLE_SA_CREDENTIALS="google service account credentials"\nUSER="username"\nPASSWORD="password"\nSAVE_STATEMENTS="save downloaded statements (default=True)"\nMAX_DOWNLOADS="maximum number of statements to download"\n```\n\n## Contributors\n\n* Ryan Fobel ([@ryanfobel](https://github.com/ryanfobel))\n',
    'author': 'Ryan Fobel',
    'author_email': 'ryan@fobel.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
